@@ -6,34 +6,64 @@
 #' @export
 setClass('mutex')
 
+#' Lock and Unlock a Mutex
+#' 
+#' @description The \code{lock} and \code{unlock} functions allow a user to
+#' specify exclusive or shared access to a resource.
+#' @param m a mutex.
+#' @param ... options associated with the mutex being used including
+#' \code{block} which forces the mutex to return immediately after trying
+#' to acquire a lock
+#' @details A call to \code{lock} gives exclusive access to a resource; no other
+#' mutex may acquire a lock.  A call to to \code{lock.shared} allows other
+#' mutexes to acquire a shared lock on the resource.  When shared lock is
+#' called while a exclusive lock has been acquired, the shared lock will
+#' block until the exclusive lock is release.  Likewise, if an exclusive lock
+#' is called while a shared lock has been acquired, the exclusive lock will
+#' block until the shared lock is released.
+#' @return The function returns \code{TRUE} if the lock is successfully 
+#' called and \code{FALSE} otherwise
+#' @examples 
+#' m = boost.mutex()
+#' lock(m)
+#' # Some code that needs to be synchronized...
+#' unlock(m)
+#' @rdname lock-and-unlock-methods
+#' @aliases unlock.shared,boost.mutex-method
 #' @export
 setGeneric('lock', function(m, ...) standardGeneric('lock'))
 
+#' @rdname lock-and-unlock-methods
 #' @export
 setGeneric('lock.shared', function(m, ...) standardGeneric('lock.shared'))
 
+#' @rdname lock-and-unlock-methods
 #' @export
 setGeneric('unlock', function(m, ...) standardGeneric('unlock'))
 
+#' @rdname lock-and-unlock-methods
+#' @export
+setGeneric('unlock.shared', function(m, ...) standardGeneric('unlock.shared'))
+
 #' @export
 setClass('boost.mutex', contains='mutex', 
   representation(isRead='logical', mutexInfoAddr='externalptr'))
 
-#' @title Is it a read (shared) mutex?
+#' @title Is it a shared mutex?
 #' 
-#' @description Tells the user if a mutex is a read (shared) mutex. If it is 
+#' @description Tells the user if a mutex is a shared mutex. If it is 
 #' not then it must be a write (exclusive) mutex.
 #' @docType methods
-#' @rdname read-methods
+#' @rdname shared-methods
 #' @param m the mutex 
-#' @return TRUE if the mutex is read (shared), FALSE otherwise.
+#' @return TRUE if the mutex is shared, FALSE otherwise.
 #' @export
-setGeneric('read', function(m) standardGeneric('read'))
+setGeneric('shared', function(m) standardGeneric('shared'))
 
-#' @rdname read-methods
-#' @aliases read,boost.mutex-method
+#' @rdname shared-methods
+#' @aliases shared,boost.mutex-method
 #' @export
-setMethod('read', signature(m='boost.mutex'), function(m) 
+setMethod('shared', signature(m='boost.mutex'), function(m) 
   IsRead(m@mutexInfoAddr))
 
 #' @export
@@ -43,8 +73,8 @@ setMethod('lock', signature(m='boost.mutex'),
     block = match.call()[['block']]
     if (is.null(block)) block=TRUE
     if (!is.logical(block)) stop('The block argument should be logical')
-    block_call = ifelse(block, boost_lock, boost_try_lock)
-    block_call(m@mutexInfoAddr)
+    if (block) boost_lock(m@mutexInfoAddr)
+    else boost_try_lock(m@mutexInfoAddr)
   })
 
 #' @export
@@ -54,16 +84,21 @@ setMethod('lock.shared', signature(m='boost.mutex'),
     block = match.call()[['block']]
     if (is.null(block)) block=TRUE
     if (!is.logical(block)) stop('The block argument should be logical')
-    block_call = ifelse(block, boost_lock_shared, boost_try_lock_shared)
-    block_call(m@mutexInfoAddr)
+    if (block) boost_lock_shared(m@mutexInfoAddr)
+    else boost_try_lock_shared(m@mutexInfoAddr)
+  })
+
+#' @aliases unlock,boost.mutex-method unlock.shared,boost.mutex-method
+#' @export
+setMethod('unlock', signature(m='boost.mutex'),
+  function(m, ...) {
+    boost_unlock(m@mutexInfoAddr)
   })
 
 #' @export
-setMethod('unlock', signature(m='boost.mutex'),
-  function(m, ...)
-  {
-    block_call = ifelse(read(m), boost_unlock_shared, boost_unlock)
-    block_call(m@mutexInfoAddr)
+setMethod("unlock.shared", signature(m='boost.mutex'),
+  function(m, ...) {
+    boost_unlock_shared(m@mutexInfoAddr)
   })
 
 #' @export
@@ -90,23 +125,31 @@ setMethod('is.timed', signature(m='boost.mutex'),
     return(!is.null(timeout(m)))
   })
 
+#' @importFrom uuid UUIDgenerate
 #' @export
-boost.mutex=function(sharedName=NULL, timeout=NULL)
+boost.mutex=function(sharedName=NULL, timeout=NULL, create=TRUE)
 {
   isRead = TRUE
-  if (is.null(sharedName)) 
-  {
-    sharedName = uuid()
-  }
   if (!is.null(timeout) && !is.numeric(timeout))
-  {
     stop("The timeout parameter must be numeric.")
-  }
   if (is.numeric(timeout) && timeout <= 0)
-  {
     stop("You must specify a timeout greater than zero.")
-  }
-  mutexInfoAddr=CreateBoostMutexInfo(sharedName, as.double(timeout))
+
+  mutexInfoAddr = try({
+    if (create) {
+      if (is.null(sharedName)) {
+        sharedName <- UUIDgenerate()
+        # Darwin's shared resources are only 5 characters long.
+        if (Sys.info()['sysname'] == "Darwin") {
+          sharedName <- substr(sharedName, 1, 5)
+        }
+      }
+      CreateBoostMutexInfo(sharedName, as.double(timeout))
+    }
+    else
+      AttachBoostMutexInfo(sharedName, as.double(timeout))
+  })
+  
   return(new('boost.mutex', isRead=isRead, mutexInfoAddr=mutexInfoAddr))
 }
 
@@ -144,8 +187,15 @@ setMethod('description', signature(x='descriptor'),
 #' @export
 setClass('boost.mutex.descriptor', contains='descriptor')
 
+#' @title Describe the boost.mutex object
+#' 
+#' @description The information required to ``attach'' to an existing
+#' mutex object.
+#' @param x the boost mutex object to describe.
 #' @importFrom bigmemory.sri describe
 #' @import methods
+#' @rdname describe-methods
+#' @aliases describe,boost.mutex-method
 #' @export
 setMethod('describe', signature(x='boost.mutex'),
   function(x)
@@ -207,7 +257,7 @@ setMethod('attach.mutex', signature(obj='boost.mutex.descriptor'),
   {
     desc = description(obj)
     return(boost.mutex(sharedName = desc$shared.name,
-      timeout = desc$timeout))
+      timeout = desc$timeout, FALSE))
   })
 
 
